package explorviz.live_trace_processing.filter.reconstruction;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import explorviz.live_trace_processing.Constants;
import explorviz.live_trace_processing.reader.TimeProvider;
import explorviz.live_trace_processing.record.event.AbstractAfterEventRecord;
import explorviz.live_trace_processing.record.event.AbstractAfterFailedEventRecord;
import explorviz.live_trace_processing.record.event.AbstractBeforeEventRecord;
import explorviz.live_trace_processing.record.event.AbstractEventRecord;
import explorviz.live_trace_processing.record.trace.RuntimeStatisticInformation;
import explorviz.live_trace_processing.record.trace.Trace;

class TraceReconstructionBuffer {
	private final List<AbstractEventRecord> events = new ArrayList<AbstractEventRecord>(
			Constants.TRACE_RECONSTRUCTION_BUFFER_INITIAL_SIZE);

	private int openEvents;

	private volatile boolean updatedInThisPeriod = true;

	private long lastBufferInsert = -1;
	private int maxOrderIndex = -1;

	public final void insertEvent(final AbstractEventRecord event) {
		updatedInThisPeriod = true;
		setMaxOrderIndex(event);

		if (event instanceof AbstractBeforeEventRecord) {
			openEvents++;
			final AbstractBeforeEventRecord beforeEvent = (AbstractBeforeEventRecord) event;
			beforeEvent.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1, -1, -1)); // TODO
																									// if
																									// invalid
																									// trace
																									// would
																									// be
																									// null...
		} else if ((event instanceof AbstractAfterFailedEventRecord)
				|| (event instanceof AbstractAfterEventRecord)) {
			openEvents--;
		}

		events.add(event);
	}

	public boolean isUpdatedInThisPeriod() {
		return updatedInThisPeriod;
	}

	public void setUpdatedInThisPeriod(final boolean updated) {
		updatedInThisPeriod = updated;
	}

	public final long getLastBufferInsert() {
		return lastBufferInsert;
	}

	public final void updateLastBufferInsert() {
		lastBufferInsert = TimeProvider.getCurrentTimestamp();
	}

	private final int setMaxOrderIndex(final AbstractEventRecord event) {
		final int orderIndex = event.getOrderIndex();
		if (orderIndex > maxOrderIndex) {
			maxOrderIndex = orderIndex;
		}
		return orderIndex;
	}

	public final boolean isFinished() {
		return !isInvalid();
	}

	public final boolean isInvalid() {
		return ((openEvents != 0) || events.isEmpty() || ((maxOrderIndex + 1) != events.size()));
	}

	public final Trace toTrace(final boolean valid) {
		final Stack<AbstractBeforeEventRecord> stack = new Stack<AbstractBeforeEventRecord>();
		for (final AbstractEventRecord event : events) {
			if (event instanceof AbstractBeforeEventRecord) {
				final AbstractBeforeEventRecord beforeEvent = (AbstractBeforeEventRecord) event;
				stack.push(beforeEvent);
			} else if ((event instanceof AbstractAfterEventRecord)
					|| (event instanceof AbstractAfterFailedEventRecord)) {
				if (!stack.isEmpty()) {
					final AbstractBeforeEventRecord beforeEvent = stack.pop();

					final long time = event.getLoggingTimestamp()
							- beforeEvent.getLoggingTimestamp();
					beforeEvent.setRuntimeStatisticInformation(new RuntimeStatisticInformation(1,
							time, time * time));
				}
			}
		}
		return new Trace(new ArrayList<AbstractEventRecord>(events), valid);
	}
}
