package explorviz.live_trace_processing.main;

import java.io.IOException;

import explorviz.live_trace_processing.configuration.Configuration;
import explorviz.live_trace_processing.configuration.ConfigurationFactory;
import explorviz.live_trace_processing.connector.TCPConnector;
import explorviz.live_trace_processing.filter.ITraceSink;
import explorviz.live_trace_processing.writer.load_balancer.LoadBalancer;

public class WorkerStarter {

	public static void main(final String[] args) {
		final Configuration configuration = ConfigurationFactory.createSingletonConfiguration();

		final boolean isWorker = configuration
				.getBooleanProperty(ConfigurationFactory.WORKER_ENABLED);

		ITraceSink sink = null;

		if (isWorker) {
			final TCPConnector connector = new TCPConnector(
					configuration.getStringProperty(ConfigurationFactory.WRITER_TARGET_IP),
					configuration.getIntProperty(ConfigurationFactory.WRITER_TARGET_PORT, 10133),
					configuration);

			configureLoadBalancerIfEnabled(configuration, connector);
			sink = connector;
		}

		FilterConfiguration.configureAndStartFilters(configuration, sink);
	}

	private static void configureLoadBalancerIfEnabled(final Configuration configuration,
			final TCPConnector tcpConnector) {
		final boolean loadBalancerEnabled = configuration
				.getBooleanProperty(ConfigurationFactory.LOAD_BALANCER_ENABLED);

		if (loadBalancerEnabled) {
			new LoadBalancer(
					configuration.getStringProperty(ConfigurationFactory.LOAD_BALANCER_IP),
					configuration.getIntProperty(ConfigurationFactory.LOAD_BALANCER_PORT, 9999),
					configuration.getIntProperty(ConfigurationFactory.LOAD_BALANCER_WAIT_TIME,
							20000),
					configuration
							.getStringProperty(ConfigurationFactory.LOAD_BALANCER_SCALING_GROUP),
					tcpConnector);
		} else {
			try {
				tcpConnector.connect();
			} catch (final IOException e) {
				e.printStackTrace();
			}
		}
	}
}
